import edu.mit.csail.sdg.alloy4.A4Reporter;
import edu.mit.csail.sdg.alloy4.Err;
import edu.mit.csail.sdg.alloy4.ErrorWarning;
import edu.mit.csail.sdg.alloy4compiler.ast.Command;
import edu.mit.csail.sdg.alloy4compiler.ast.Module;
import edu.mit.csail.sdg.alloy4compiler.parser.CompUtil;
import edu.mit.csail.sdg.alloy4compiler.translator.A4Options;
import edu.mit.csail.sdg.alloy4compiler.translator.A4Solution;
import edu.mit.csail.sdg.alloy4compiler.translator.TranslateAlloyToKodkod;

/** Execute Alloy4 via API.*/

public final class AlloyExecuter {

    /*
     * Execute every command in every file.
     *
     * This method parses every file, then executes every command.
     * Prints "Satisfiable." or "Unsatisfiable.", the number of clauses and variables,
     *  and then the complete Alloy solution.
     */
    public static void main(String[] args) throws Err {

	// Create reporter that displays clauses and variables.
	A4Reporter rep = new A4Reporter() {
		@Override public void solve (int primaryVars, int totalVars, int clauses) {
	                System.out.println("Vars: " + totalVars + " Clauses: " + clauses);
	                System.out.flush();
		}
        };


        for(String filename:args) {

            // Parse and typecheck model
            Module world = CompUtil.parseEverything_fromFile(rep, null, filename);

            // Choose default options for how commands should be executed
            A4Options options = new A4Options();
            options.solver = A4Options.SatSolver.SAT4J;


            for (Command command: world.getAllCommands()) 
	    {
                // Execute the command
                A4Solution ans = TranslateAlloyToKodkod.execute_command(rep, world.getAllReachableSigs(), command, options);
                

		// Print the outcome
		String answer = ans.toString();
		
		if (answer.length() > 30) {
		    System.out.println( "Satisfiable." );
		}
		else {
		    System.out.println( answer.subSequence(14, answer.length()-1) ); 
		}
		
		System.out.println();
		System.out.println(answer);
	        System.out.flush();
	    
            }
        }
    }
}
